# $Id: Preview.pm,v 1.1 2003/01/27 14:17:54 joern Exp $

#-----------------------------------------------------------------------
# Copyright (C) 2001-2003 Jrn Reder <joern AT zyn.de> and others.
# All Rights Reserved. See file COPYRIGHT for details.
# 
# This module is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::DVDRip::GUI::Preview;

use Carp;
use strict;

@Video::DVDRip::GUI::Preview::ISA = qw ( Video::DVDRip::GUI::Base );

use Video::DVDRip::TranscodeRC;

sub frame_nr			{ shift->{frame_nr}			}
sub transcode_remote		{ shift->{transcode_remote}		}
sub transcode_pipe		{ shift->{transcode_pipe}		}

sub set_frame_nr		{ shift->{frame_nr}		= $_[1]	}
sub set_transcode_remote	{ shift->{transcode_remote}	= $_[1]	}
sub set_transcode_pipe		{ shift->{transcode_pipe}	= $_[1]	}

sub stop_in_progress		{ shift->{stop_in_progress}		}
sub set_stop_in_progress	{ shift->{stop_in_progress}	= $_[1]	}

sub new {
	my $class = shift;
	
	my $self = {
	};
	
	return bless $self, $class;
}

sub closed { not defined shift->transcode_pipe }

sub open {
	my $self = shift;

	croak "Preview already open" if $self->set_transcode_pipe;
	
	my $socket_file = "/tmp/tc.$$.sock";
	
	# start transcode
	
	my $title = $self->comp('project')->selected_title;
	
	my ($orig_start, $orig_end) = (
		$title->tc_start_frame,
		$title->tc_end_frame,
	);
	
	$title->set_tc_start_frame ( $title->tc_preview_start_frame );
	$title->set_tc_end_frame   ( $title->tc_preview_end_frame );
	
	my $command = $title->get_transcode_command ( pass => 1 );
	
	$title->set_tc_start_frame ( $orig_start );
	$title->set_tc_end_frame   ( $orig_end );

	$command =~ s/&& echo DVDRIP_SUCCESS//;
	$command =~ s/-y\s+[^\s]+//;	# remove target codecs
	$command =~ s/-o\s+[^\s]+//;	# remove output file
	$command =~ s/-J\s+[^\s]+//g;	# remove all filters
	
	$command .= " -J pv=cache=".$title->tc_preview_buffer_frames;
	$command .= " --socket $socket_file";

	my $transcode_pipe = Video::DVDRip::GUI::Pipe->new (
		command 	=> $command,
		need_output	=> 0,
		cb_line_read	=> sub {
			my ($line) = @_;
			if ( $line =~ /\[(\d{6}-)?(\d+)\]/ ) {
				$self->set_frame_nr ( $1 );
			}
			1;
		},
		cb_finished	=> sub {
			my $stopped = $self->stop_in_progress;
			$self->close;
			$self->open if not $stopped;
			1;
		},
	);
	
	$transcode_pipe->open;
	
	$self->set_transcode_pipe ( $transcode_pipe );

	my $time = time;
	while ( not -e $socket_file ) {
		last if time - $time >= 2;
	}

	if ( not -e $socket_file ) {
		$self->stop;
		$self->close;
		croak "msg:Couldn't start transcode.";
	}

	# start remote control
	
	my $transcode_remote = Video::DVDRip::TranscodeRC->new (
		socket_file => $socket_file,
		error_cb => sub {
			my ($line) = @_;
			$self->long_message_window (
				message => "Error executing a transcode socket command:\n\n".
					   $line
			);
			1;
		}
	);

	$transcode_remote->connect;
	
	my $socket_fileno = $transcode_remote->socket->fileno;
	
	my $gdk_input = Gtk::Gdk->input_add (
		$socket_fileno,
		'read', sub {
			my $rc = $transcode_remote->receive;
			if ( defined $rc ) {
				$self->process_transcode_remote_output (
					line => $rc,
				) if $rc ne '';
			}
			1;
		}
	);

	$transcode_remote->set_closed_cb (sub {
		Gtk::Gdk->input_remove( $gdk_input );
	} );
	
	$self->set_transcode_remote ( $transcode_remote );
	
	$self->apply_filter_settings;
	
	1;
}

sub apply_filter_settings {
	my $self = shift;

	my $title = $self->comp('project')->selected_title;

	my $config_strings =
		$title->tc_filter_settings
		      ->get_filter_config_strings;

	my $max_frames_needed =
		$title->tc_filter_settings
		      ->get_max_frames_needed,

	my $transcode_remote = $self->transcode_remote;

	foreach my $config ( @{$config_strings} ) {
		if ( not $config->{enabled} ) {
			$transcode_remote->disable_filter (
				filter => $config->{filter}
			);
			next;
		}
		$transcode_remote->config_filter (
			filter  => $config->{filter},
			options => $config->{options},
		);
		$transcode_remote->enable_filter (
			filter => $config->{filter}
		);
	}
	
	$transcode_remote->preview (
		command => "draw",
		options => $max_frames_needed,
	) if $transcode_remote->paused;
	
	1;
}	

sub stop {
	my $self = shift;
	
	$self->set_stop_in_progress ( 1 );
	
	$self->transcode_remote->quit if $self->transcode_remote;
	$self->transcode_pipe->cancel if $self->transcode_pipe;
	
	1;
}

sub close {
	my $self = shift;
	
	$self->transcode_remote->disconnect;

	$self->set_frame_nr		(undef);
	$self->set_transcode_pipe	(undef);
	$self->set_transcode_remote	(undef);
	
	$self->set_stop_in_progress ( 0 );
	
	1;
}

sub pause {
	my $self = shift;
	
	$self->transcode_remote->preview (
		command => "pause",
	);

	return $self->transcode_remote->paused;
}

sub process_transcode_remote_output {
	my $self = shift;
	my %par = @_;
	my  ($line) =
	@par{'line'};
	
	print "GOT: line='$line'\n";
	
	1;
}

1;

